@@ -5,8 +5,10 @@ non-isomorphic multigraphs, which linearly span the space of infrared and
 collinear (IRC) safe observables.
 
 An EFP, indexed by a multigraph $G$, takes the following form:
-$$\text{EFP}_G=\sum_{i_1=1}^M\cdots\sum_{i_N=1}^Mz_{i_1}\cdots z_{i_N}
-\prod_{(k,\ell)\in G}\theta_{i_ki_\ell}$$
+
+\[\text{EFP}_G=\sum_{i_1=1}^M\cdots\sum_{i_N=1}^Mz_{i_1}\cdots z_{i_N}
+\prod_{(k,\ell)\in G}\theta_{i_ki_\ell}\]
+
 where $z_i$ is a measure of the energy of particle $i$ and $\theta_{ij}$ is a
 measure of the angular separation between particles $i$ and $j$. The specific
 choices for "energy" and "angular" measure depend on the collider context and
@@ -41,7 +43,7 @@ class EFP(EFPBase):
     """A class for representing and computing a single EFP."""
 
     # EFP(edges, measure='hadr', beta=1, kappa=1, normed=None, coords=None,
-    #     check_input=True, np_optimize=True)
+    #            check_input=True, np_optimize=True)
     def __init__(self, edges, weights=None, efpset_args=None, np_optimize=True, **kwargs):
         r"""Since a standalone EFP defines and holds a `Measure` instance, all
         `Measure` keywords are accepted.
