@@ -6,66 +6,363 @@ r"""# Energy Mover's Distance
 </video>
 <br>
 
-The Energy Mover's Distance (EMD), also known as the Earth Mover's 
-Distance, is a metric between particle collider events introduced in
-[1902.02346](https://arxiv.org/abs/1902.02346). This submodule contains
-convenient functions for computing EMDs between individual events and
-collections of events. The core of the computation is done using the
-[Python Optimal Transport (POT)](https://pot.readthedocs.io) library,
-which must be installed in order to use this submodule.
+The Energy Mover's Distance (EMD), also known as the Earth Mover's Distance, is
+a metric between particle collider events introduced in [1902.02346](https://
+arxiv.org/abs/1902.02346). This submodule contains convenient functions for
+computing EMDs between individual events and collections of events. The core of
+the computation is handled by either the [Wasserstein](https://github.com/
+pkomiske/Wasserstein) library or the [Python Optimal Transport (POT)](https://
+pot.readthedocs.io) library, one of which must be installed in order to use this
+submodule.
+
+From Eqs. (1.2) and (1.3) in [2004.04159](https://arxiv.org/abs/2004.04159), the
+EMD between two events is the minimum ''work'' required to rearrange one event
+$\mathcal E$ into the other $\mathcal E'$ by movements of energy $f_{ij}$ from
+particle $i$ in one event to particle $j$ in the other:
+
+\[\text{EMD}_{\beta,R}(\mathcal E,\mathcal E^\prime)=\min_{\{f_{ij}\ge0\}}\sum_{i=1}^M\sum_{j=1}^{M'}f_{ij}\left(\frac{
+\theta_{ij}}{R}\right)^\beta + \left|\sum_{i=1}^ME_i-\sum_{j=1}^{M'}E^\prime_j
+\right|,\]
+
+\[\sum_{j=1}^{M'}f_{ij}\le E_i, \quad \sum_{i=1}^Mf_{ij}\le E^\prime_j,
+\quad\sum_{i=1}^M\sum_{j=1}^{M'}f_{ij}=E_\text{min},\]
 
-From Eq. 1 in [1902.02346](https://arxiv.org/abs/1902.02346), the EMD between
-two events is the minimum ''work'' required to rearrange one event $\mathcal E$
-into the other $\mathcal E'$ by movements of energy $f_{ij}$ from particle $i$ 
-in one event to particle $j$ in the other:
-$$
-\text{EMD}(\mathcal E,\mathcal E^\prime)=\min_{\{f_{ij}\}}\sum_{ij}f_{ij}\frac{
-\theta_{ij}}{R} + \left|\sum_iE_i-\sum_jE^\prime_j\right|,\\
-f_{ij}\ge 0, \quad \sum_jf_{ij}\le E_i, \quad \sum_if_{ij}\le E^\prime_j, \quad
-\sum_{ij}f_{ij}=E_\text{min},
-$$
 where $E_i,E^\prime_j$ are the energies of the particles in the two events, 
 $\theta_{ij}$ is an angular distance between particles, and 
-$E_\text{min}=\min\left(\sum_iE_i,\,\sum_jE^\prime_j\right)$ is the smaller
-of the two total energies. In a hadronic context, transverse momenta are used
-instead of energies.
+$E_\text{min}=\min\left(\sum_{i=1}^ME_i,\,\sum_{j=1}^{M'}E^\prime_j\right)$ is
+the smaller of the two total energies. In a hadronic context, transverse momenta
+are used instead of energies.
 """
 from __future__ import absolute_import, division, print_function
 
 import itertools
 import multiprocessing
-import sys
 import time
+import warnings
 
 import numpy as np
 
-ot = True
+# try to import POT
 try:
+    import ot
     from ot.lp import emd_c, check_result
     from scipy.spatial.distance import _distance_wrap # ot imports scipy anyway
 except:
     ot = False
 
-from energyflow.utils import create_pool, p4s_from_ptyphims
+# try to import Wasserstein
+try:
+    import wasserstein
+except:
+    wasserstein = False
 
-__all__ = ['emd', 'emds']
+from energyflow.utils import create_pool, kwargs_check, p4s_from_ptyphims
 
-# replace public functions with those issuing simple errors
-if not ot:
+__all__ = [
+    'emd', 'emds',
+    'emd_wasserstein', 'emds_wasserstein',
+    'emd_pot', 'emds_pot'
+]
 
-    def emd(*args, **kwargs):
-        raise NotImplementedError("emd currently requires module 'ot', which is unavailable")
+#########################
+# DOCUMENTATION FUNCTIONS
+#########################
 
-    def emds(*args, **kwargs):
-        raise NotImplementedError("emd currently requires module 'ot', which is unavailable")
+# emd(*args, **kwargs)
+def emd4doc():
+    """Computes the EMD between two events. The `emd` function is set equal to
+    [`emd_wasserstein`](#emd_wasserstein) or [`emd_pot`](#emd_pot), with the
+    former preferred unless the Wasserstein library is not available.
+    """
 
-# the actual functions for this module
+    pass
+
+# emds(*args, **kwargs)
+def emds4doc():
+    """Computes the EMDs between collections of events. The `emds` function is
+    set equal to [`emds_wasserstein`](#emds_wasserstein) or
+    [`emds_pot`](#emds_pot), with the former preferred unless the Wasserstein
+    library is not available.
+    """
+
+    pass
+
+#######################
+# WASSERSTEIN FUNCTIONS
+#######################
+
+# EMD implementations using Wasserstein
+if wasserstein:
+
+    # global wasserstein EMD object to carry out computations
+    _EMD = wasserstein.EMD()
+
+    # emd_wasserstein(ev0, ev1, dists=None, R=1.0, beta=1.0, norm=False, gdim=2, mask=False,
+    #                           return_flow=False, do_timing=False,
+    #                           n_iter_max=100000,
+    #                           epsilon_large_factor=10000.0, epsilon_small_factor=1.0)
+    def emd_wasserstein(ev0, ev1, dists=None, R=1.0, beta=1.0, norm=False, gdim=2, mask=False,
+                                  return_flow=False, do_timing=False,
+                                  n_iter_max=100000,
+                                  epsilon_large_factor=10000.0, epsilon_small_factor=1.0,
+                                  **kwargs):
+        r"""Compute the EMD between two events using the Wasserstein library.
+
+        **Arguments**
+
+        - **ev0** : _numpy.ndarray_
+            - The first event, given as a two-dimensional array. The event is 
+            assumed to be an `(M,1+gdim)` array of particles, where `M` is the 
+            multiplicity and `gdim` is the dimension of the ground space in 
+            which to compute euclidean distances between particles (as specified
+            by the `gdim` keyword argument). The zeroth column is the weights of
+            the particles, typically their energies or transverse momenta. For
+            typical hadron collider jet applications, each particle will be of
+            the form `(pT,y,phi)` where  `y` is the rapidity and `phi` is the
+            azimuthal angle. If `dists` are provided, then the columns after the
+            zeroth are ignored; alternatively a one-dimensional array consisting
+            of just the particle weights may be passed in this case.
+        - **ev1** : _numpy.ndarray_
+            - The other event, same format as `ev0`.
+        - **dists** : _numpy.ndarray_
+            - A distance matrix between particles in `ev0` and `ev1`. If `None`,
+            then the columns of the events after the zeroth are taken to be
+            coordinates and the `gdim`-dimensional Euclidean distance is used.
+        - **R** : _float_
+            - The R parameter in the EMD definition that controls the relative 
+            importance of the two terms. Must be greater than or equal to half 
+            of the maximum ground distance in the space in order for the EMD 
+            to be a valid metric satisfying the triangle inequality.
+        - **beta** : _float_
+            - The angular weighting exponent. The internal pairwsie distance
+            matrix is raised to this power prior to solving the optimal
+            transport problem.
+        - **norm** : _bool_
+            - Whether or not to normalize the particle weights to sum to one
+            prior to computing the EMD.
+        - **gdim** : _int_
+            - The dimension of the ground metric space. Useful for restricting
+            which dimensions are considered part of the ground space when using
+            the internal euclidean distances between particles. Has no effect if
+            `dists` are provided.
+        - **return_flow** : _bool_
+            - Whether or not to return the flow matrix describing the optimal 
+            transport found during the computation of the EMD. Note that since
+            the second term in Eq. 1 is implemented by including an additional 
+            particle in the event with lesser total weight, this will be
+            reflected in the flow matrix.
+        - **mask** : _bool_
+            - If `True`, masks out particles farther than `R` away from the
+            origin. Has no effect if `dists` are provided.
+        - **n_iter_max** : _int_
+            - Maximum number of iterations for solving the optimal transport 
+            problem.
+        - **epsilon_large_factor** : _float_
+            - Controls some tolerances in the optimal transport solver. This
+            value is multiplied by the floating points epsilon (around 1e-16 for
+            64-bit floats) to determine the actual tolerance.
+        - **epsilon_small_factor** : _float_
+            - Analogous to `epsilon_large_factor` but used where the numerical
+            tolerance can be stricter.
+
+        **Returns**
+
+        - _float_
+            - The EMD value.
+        - [_numpy.ndarray_], optional
+            - The flow matrix found while solving for the EMD. The `(i,j)`th 
+            entry is the amount of `pT` that flows between particle i in `ev0`
+            and particle j in `ev1`.
+        """
+
+        # warn about old kwargs
+        old_kwargs = {'measure', 'coords', 'periodic_phi', 'phi_col', 'empty_policy'}
+        kwargs_check('emd_wasserstein', kwargs, old_kwargs)
+        for k in kwargs:
+            warnings.warn("Keyword argument '{}' has no effect on `emd_wasserstein`.".format(k)
+                          + " Use `emd_pot` if you need previous functionality.")
+
+        # set options
+        _EMD.set_R(R)
+        _EMD.set_beta(beta)
+        _EMD.set_norm(norm)
+        _EMD.set_network_simplex_params(n_iter_max, epsilon_large_factor, epsilon_small_factor)
+
+        # run using euclidean distances
+        if dists is None:
+            ev0, ev1 = np.atleast_2d(ev0)[:,:gdim+1], np.atleast_2d(ev1)[:,:gdim+1]
+
+            # mask out particles
+            if mask:
+                R2 = R*R
+                ev0, ev1 = ev0[np.sum(ev0**2, axis=1) <= R2], ev1[np.sum(ev1**2, axis=1) <= R2]
+
+            # evaluate EMD
+            emd = _EMD(ev0[:,0], ev0[:,1:], ev1[:,0], ev1[:,1:])
+
+        # run using custom distances
+        else:
+
+            # if events are 2d, extract weights as just the first column
+            if ev0.ndim == 2:
+                ev0 = ev0[:,0]
+            if ev1.ndim == 2:
+                ev1 = ev1[:,0]
+
+            # evaluate EMD
+            emd = _EMD(ev0, ev1, dists)
+
+        # get flows if requested
+        if return_flow:
+            flows = _EMD.flows()
+
+        if return_flow:
+            return emd, flows
+        else:
+            return emd
+
+    # emds_wasserstein(events0, events1=None, R=1.0, beta=1.0, norm=False, gdim=2, mask=False,
+    #                                         external_emd_handler=None,
+    #                                         n_jobs=-1, print_every=0, verbose=0,
+    #                                         throw_on_error=True, n_iter_max=100000,
+    #                                         epsilon_large_factor=10000.0,
+    #                                         epsilon_small_factor=1.0)
+    def emds_wasserstein(events0, events1=None, R=1.0, beta=1.0, norm=False, gdim=2, mask=False,
+                                                external_emd_handler=None,
+                                                n_jobs=-1, print_every=0, verbose=0,
+                                                throw_on_error=True, n_iter_max=100000,
+                                                epsilon_large_factor=10000.0, epsilon_small_factor=1.0,
+                                                **kwargs):
+        r"""Compute the EMDs between collections of events. This can be used to
+        compute EMDs between all pairs of events in a set or between events in
+        two different sets.
+
+        **Arguments**
+
+        - **events0** : _list_
+            - Iterable collection of events. Each event is assumed to be an 
+            `(M,1+gdim)` array of particles, where `M` is the multiplicity and
+            `gdim` is the dimension of the ground space in which to compute
+            euclidean distances between particles (as specified by the `gdim`
+            keyword argument). The zeroth column is the weights of the
+            particles, typically their energies or transverse momenta. For
+            typical hadron collider jet applications, each particle will be of
+            the form `(pT,y,phi)` where  `y` is the rapidity and `phi` is the
+            azimuthal angle. If `dists` are provided, then the columns after the
+            zeroth are ignored; alternatively a one-dimensional array consisting
+            of just the particle weights may be passed in this case.
+        - **events1** : _list_ or `None`
+            - Iterable collection of events in the same format as `events0`, or
+            `None`. If the latter, the pairwise distances between events in
+            `events0` will be computed and the returned matrix will be
+            symmetric.
+       - **R** : _float_
+            - The R parameter in the EMD definition that controls the relative 
+            importance of the two terms. Must be greater than or equal to half 
+            of the maximum ground distance in the space in order for the EMD 
+            to be a valid metric satisfying the triangle inequality.
+        - **norm** : _bool_
+            - Whether or not to normalize the particle weights to sum to one
+            prior to computing the EMD.
+        - **beta** : _float_
+            - The angular weighting exponent. The internal pairwsie distance
+            matrix is raised to this power prior to solving the optimal
+            transport problem.
+         - **gdim** : _int_
+            - The dimension of the ground metric space. Useful for restricting
+            which dimensions are considered part of the ground space when using
+            the internal euclidean distances between particles.
+        - **mask** : _bool_
+            - If `True`, ignores particles farther than `R` away from the
+            origin.
+        - **external_emd_handler** : _wasserstein.ExternalEMDHandler_
+            - An instance of an external EMD handler from the wasserstein
+            module, e.g. `CorrelationDimension`.
+        - **n_jobs** : _int_ or `None`
+            - The number of cpu cores to use. A value of `None` or `-1` will use
+            as many threads as there are CPUs on the machine.
+        - **print_every** : _int_
+            - The number of computations to do in between printing the
+            progress. Even if the verbosity level is zero, this still plays a
+            role in determining when the worker threads report the results
+            back to the main thread and check for interrupt signals.
+        - **verbose** : _int_
+            - Controls the verbosity level. A value greater than `0` will print
+            the progress of the computation at intervals specified by
+            `print_every`.
+        - **throw_on_error** : _bool_
+            - Whether or not to raise an exception when an issue is encountered.
+            Can be useful when debugging.
+        - **n_iter_max** : _int_
+            - Maximum number of iterations for solving the optimal transport 
+            problem.
+        - **epsilon_large_factor** : _float_
+            - Controls some tolerances in the optimal transport solver. This
+            value is multiplied by the floating points epsilon (around 1e-16 for
+            64-bit floats) to determine the actual tolerance.
+        - **epsilon_small_factor** : _float_
+            - Analogous to `epsilon_large_factor` but used where the numerical
+            tolerance can be stricter.
+
+        **Returns**
+
+        - _numpy.ndarray_
+            - The EMD values as a two-dimensional array, except if an external
+            EMD handler was provided, in which case no value is returned. If
+            `events1` was `None`, then the shape will be `(len(events0),
+            len(events0))` and the array will be symmetric, otherwise it will
+            have shape `(len(events0), len(events1))`.
+        """
+
+        # warn about old kwargs
+        old_kwargs = {'X0', 'X1', 'measure', 'coords', 'periodic_phi', 'phi_col', 'empty_policy'}
+        kwargs_check('emds_wasserstein', kwargs, old_kwargs)
+        for k in kwargs:
+            warnings.warn("Keyword argument '{}' has no effect on `emds_wasserstein`.".format(k)
+                          + " Use `emds_pot` if you need previous functionality.")
+
+        # determine number of threads to use
+        if n_jobs is None or n_jobs == -1:
+            n_jobs = multiprocessing.cpu_count() or 1
+
+        # create object
+        pairwise_emd = wasserstein.PairwiseEMD(R, beta, norm, n_jobs, print_every, bool(verbose),
+                                               throw_on_error=throw_on_error,
+                                               n_iter_max=n_iter_max,
+                                               epsilon_large_factor=epsilon_large_factor,
+                                               epsilon_small_factor=epsilon_small_factor)
+        if verbose > 0:
+            print(pairwise_emd)
+
+        # set handler if given
+        if external_emd_handler is not None:
+            pairwise_emd.set_external_emd_handler(external_emd_handler)
+
+        # run computation
+        pairwise_emd(events0, events1, gdim, mask)
+
+        # return flows if handler not provided
+        if external_emd_handler is None:
+            return pairwise_emd.emds()
+
+    # prefer wasserstein implementations
+    emd, emds = emd_wasserstein, emds_wasserstein
+
+# Wasserstein not available
+else:
+    message = "'wasserstein' not available"
+    def emd_wasserstein(*args, **kwargs):
+        raise NotImplementedError(message)
+    def emds_wasserstein(*args, **kwargs):
+        raise NotImplementedError(message)
+
+###############
+# POT FUNCTIONS
+###############
+
+# EMD implementations using POT
 if ot:
 
-##################
-# HELPER FUNCTIONS
-##################
-
     # parameter checks
     def _check_params(norm, gdim, phi_col, measure, coords, empty_policy):
 
@@ -241,14 +538,14 @@ if ot:
 
         return cost * rescale
 
-##################
-# PUBLIC FUNCTIONS
-##################
-
-    def emd(ev0, ev1, R=1.0, norm=False, beta=1.0, measure='euclidean', coords='hadronic',
-                      return_flow=False, gdim=None, mask=False, n_iter_max=100000,
-                      periodic_phi=False, phi_col=2, empty_policy='error'):
-        r"""Compute the EMD between two events.
+    # emd_pot(ev0, ev1, R=1.0, norm=False, beta=1.0, measure='euclidean', coords='hadronic',
+    #                   return_flow=False, gdim=None, mask=False, n_iter_max=100000,
+    #                   periodic_phi=False, phi_col=2, empty_policy='error')
+    def emd_pot(ev0, ev1, R=1.0, norm=False, beta=1.0, measure='euclidean', coords='hadronic',
+                         return_flow=False, gdim=None, mask=False, n_iter_max=100000,
+                         periodic_phi=False, phi_col=2, empty_policy='error'):
+        r"""Compute the EMD between two events using the Python Optimal
+        Transport library.
 
         **Arguments**
 
@@ -263,15 +560,15 @@ if ot:
             particle will be of the form `(pT,y,phi)` where  `y` is the rapidity
             and `phi` is the azimuthal angle.
         - **ev1** : _numpy.ndarray_
-            - The other event, same format as **ev0**.
+            - The other event, same format as `ev0`.
         - **R** : _float_
             - The R parameter in the EMD definition that controls the relative 
             importance of the two terms. Must be greater than or equal to half 
             of the maximum ground distance in the space in order for the EMD 
-            to be a valid metric.
+            to be a valid metric satisfying the triangle inequality.
         - **beta** : _float_
             - The angular weighting exponent. The internal pairwsie distance
-            matrix is raised to this power priot to solving the optimal
+            matrix is raised to this power prior to solving the optimal
             transport problem.
         - **norm** : _bool_
             - Whether or not to normalize the pT values of the events prior to 
@@ -395,11 +692,15 @@ if ot:
         else:
             return cost * rescale
 
-    def emds(X0, X1=None, R=1.0, norm=False, beta=1.0, measure='euclidean', coords='hadronic',
-                          gdim=None, mask=False, n_iter_max=100000, 
-                          periodic_phi=False, phi_col=2, empty_policy='error',
-                          n_jobs=None, verbose=0, print_every=10**6):
-        r"""Compute the EMD between collections of events. This can be used to
+    # emds_pot(X0, X1=None, R=1.0, norm=False, beta=1.0, measure='euclidean', coords='hadronic',
+    #              gdim=None, mask=False, n_iter_max=100000, 
+    #              periodic_phi=False, phi_col=2, empty_policy='error',
+    #              n_jobs=None, verbose=0, print_every=10**6)
+    def emds_pot(X0, X1=None, R=1.0, norm=False, beta=1.0, measure='euclidean', coords='hadronic',
+                             gdim=None, mask=False, n_iter_max=100000, 
+                             periodic_phi=False, phi_col=2, empty_policy='error',
+                             n_jobs=None, verbose=0, print_every=10**6):
+        r"""Compute the EMDs between collections of events. This can be used to
         compute EMDs between all pairs of events in a set or between events in
         two different sets.
 
@@ -423,13 +724,13 @@ if ot:
             - The R parameter in the EMD definition that controls the relative 
             importance of the two terms. Must be greater than or equal to half 
             of the maximum ground distance in the space in order for the EMD 
-            to be a valid metric.
+            to be a valid metric satisfying the triangle inequality.
         - **norm** : _bool_
             - Whether or not to normalize the pT values of the events prior to 
             computing the EMD.
         - **beta** : _float_
             - The angular weighting exponent. The internal pairwsie distance
-            matrix is raised to this power priot to solving the optimal
+            matrix is raised to this power prior to solving the optimal
             transport problem.
         - **measure** : _str_
             - Controls which metric is used to calculate the ground distances
@@ -524,7 +825,7 @@ if ot:
         # handle kwarg options
         if isinstance(print_every, float):
             print_every = int(npairs*print_event)
-        if n_jobs is None:
+        if n_jobs is None or n_jobs == -1:
             n_jobs = multiprocessing.cpu_count() or 1
 
         # setup container for EMDs
@@ -594,3 +895,26 @@ if ot:
             emds += emds.T
 
         return emds
+
+    # set emd and emds to pot functions if wasserstein not available
+    if not wasserstein:
+        emd, emds = emd_pot, emds_pot
+        warnings.warn("'wasserstein' module not available, falling back on slower POT implementation")
+
+# POT not available
+else:
+    message = "'pot' not available"
+    def emd_pot(*args, **kwargs):
+        raise NotImplementedError(message)
+    def emds_pot(*args, **kwargs):
+        raise NotImplementedError(message)
+
+    # if wasserstein also not available, emd and emds functions not available
+    if not wasserstein:
+        message = "emd module requires 'wasserstein' or 'pot', both of which are unavailable"
+        warnings.warn(message)
+
+        def emd(*args, **kwargs):
+            raise NotImplementedError(message)
+        def emds(*args, **kwargs):
+            raise NotImplementedError(message)
